import click

from pheval_lirical.post_process.post_process_results_format import post_process
from pheval_lirical.prepare.prepare_commands import prepare_commands_command


def main_():
    pass


@click.group()
def main():
    """Lirical runner."""
    pass


main.add_command(prepare_commands_command)
main.add_command(post_process)

if __name__ == "__main__":
    main()



"""LIRICAL Runner"""

from dataclasses import dataclass
from pathlib import Path

from pheval.runners.runner import PhEvalRunner

from pheval_lirical.post_process.post_process import post_process_results_format
from pheval_lirical.run.run import prepare_lirical_commands, run_lirical_local
from pheval_lirical.tool_specific_configuration_parser import LIRICALToolSpecificConfigurations


@dataclass
class LiricalPhEvalRunner(PhEvalRunner):
    """_summary_"""

    input_dir: Path
    testdata_dir: Path
    tmp_dir: Path
    output_dir: Path
    config_file: Path
    version: str

    def prepare(self):
        """prepare"""
        print("preparing")

    def run(self):
        """run"""
        print("running with lirical")
        config = LIRICALToolSpecificConfigurations.parse_obj(
            self.input_dir_config.tool_specific_configuration_options
        )
        prepare_lirical_commands(
            input_dir=self.input_dir,
            testdata_dir=self.testdata_dir,
            raw_results_dir=self.raw_results_dir,
            tool_input_commands_dir=self.tool_input_commands_dir,
            lirical_version=self.version,
            tool_specific_configurations=config,
            gene_analysis=self.input_dir_config.gene_analysis,
            variant_analysis=self.input_dir_config.variant_analysis,
        )
        run_lirical_local(
            testdata_dir=self.testdata_dir, tool_input_commands_dir=self.tool_input_commands_dir
        )

    def post_process(self):
        """post_process"""
        print("post processing")
        config = LIRICALToolSpecificConfigurations.parse_obj(
            self.input_dir_config.tool_specific_configuration_options
        )
        post_process_results_format(
            raw_results_dir=self.raw_results_dir,
            output_dir=self.output_dir,
            config=config,
            disease_analysis=self.input_dir_config.disease_analysis,
            gene_analysis=self.input_dir_config.gene_analysis,
            variant_analysis=self.input_dir_config.variant_analysis,
        )



import re
from pathlib import Path

import click
import pandas as pd
from pheval.post_processing.post_processing import (
    PhEvalDiseaseResult,
    PhEvalGeneResult,
    PhEvalVariantResult,
    calculate_end_pos,
    generate_pheval_result,
)
from pheval.utils.file_utils import files_with_suffix
from pheval.utils.phenopacket_utils import (
    GeneIdentifierUpdater,
    create_gene_identifier_map,
    create_hgnc_dict,
)


def read_lirical_result(lirical_result_path: Path) -> pd.DataFrame:
    """Read LIRICAL tsv output and return a dataframe."""
    return pd.read_csv(lirical_result_path, delimiter="\t", comment="!")


class PhEvalGeneResultFromLirical:
    def __init__(
        self,
        lirical_result: pd.DataFrame,
        gene_identifier_updator: GeneIdentifierUpdater,
    ):
        self.lirical_result = lirical_result
        self.gene_identifier_updator = gene_identifier_updator

    @staticmethod
    def obtain_lirical_gene_identifier(result: pd.Series) -> str:
        """Obtain the NCBI gene id from LIRICAL result."""
        return str(result["entrezGeneId"])

    def obtain_gene_symbol(self, result: pd.Series) -> str:
        """Obtain the gene symbol from NCBI gene."""
        gene_identifier = self.obtain_lirical_gene_identifier(result)
        return (
            self.gene_identifier_updator.obtain_gene_symbol_from_identifier(
                gene_identifier.split(":")[1]
            )
            if ":" in gene_identifier
            else gene_identifier
        )

    def obtain_gene_identifier(self, result: pd.Series) -> str:
        """Obtain the ensembl gene identifier from gene symbol."""
        gene_symbol = self.obtain_gene_symbol(result)
        return self.gene_identifier_updator.find_identifier(gene_symbol)

    @staticmethod
    def obtain_score(result: pd.Series) -> float:
        """Obtain score from result."""
        if result["compositeLR"] == "-∞":
            return float("-inf")
        return float(result["compositeLR"])

    def extract_pheval_requirements(self) -> [PhEvalGeneResult]:
        """Extract data required to produce PhEval gene output."""
        simplified_gene_results = []
        for _index, result in self.lirical_result.iterrows():
            simplified_gene_results.append(
                PhEvalGeneResult(
                    gene_symbol=self.obtain_gene_symbol(result),
                    gene_identifier=self.obtain_gene_identifier(result),
                    score=self.obtain_score(result),
                )
            )
        return simplified_gene_results


class PhEvalVariantResultFromLirical:
    def __init__(self, lirical_result: pd.DataFrame):
        self.lirical_result = lirical_result
        self.reg = re.compile(r"(?P<numbers>\d*)(?P<rest>.*)")

    @staticmethod
    def obtain_score(lirical_result_entry: pd.Series) -> float:
        """Obtain score from result."""
        if lirical_result_entry["compositeLR"] == "-∞":
            return float("-inf")
        return float(lirical_result_entry["compositeLR"])

    @staticmethod
    def obtain_variants(lirical_result_entry: pd.Series) -> str:
        """Obtain all variants listed in a result."""
        return lirical_result_entry["variants"]

    def split_variants(self, lirical_result_entry) -> [str]:
        """Split variants contained in a single result row."""
        variants = self.obtain_variants(lirical_result_entry)
        return [x.strip() for x in variants.split(";")]

    @staticmethod
    def get_variant_string(variant: str) -> str:
        """Obtain variant string."""
        return variant.split(" ")[0]

    def obtain_chromosome(self, variant_str: str) -> str:
        """Obtain chromosome from variant string."""
        variant = self.get_variant_string(variant_str)
        return variant.split(":")[0]

    def obtain_start(self, variant_str: str) -> int:
        """Obtain start position from variant string."""
        variant = self.get_variant_string(variant_str)
        return int(self.reg.search(variant.split(":")[1]).group("numbers"))

    def obtain_ref(self, variant_str: str) -> str:
        """Obtain reference allele from variant string."""
        variant = self.get_variant_string(variant_str)
        return self.reg.search(variant.split(":")[1]).group("rest").split(">")[0]

    def obtain_end(self, variant_str: str) -> int:
        """Obtain end position from variant string."""
        variant = self.get_variant_string(variant_str)
        return calculate_end_pos(self.obtain_start(variant), self.obtain_ref(variant))

    def obtain_alt(self, variant_str: str) -> str:
        """Obtain alternate allele from variant string."""
        variant = self.get_variant_string(variant_str)
        return self.reg.search(variant.split(":")[1]).group("rest").split(">")[1]

    def extract_pheval_requirements(self) -> PhEvalVariantResult:
        """Extract data required to produce PhEval variant output."""
        simplified_variant_results = []
        for _index, result in self.lirical_result.iterrows():
            variant_results = self.split_variants(result)
            for variant_result in variant_results:
                if variant_result != "":
                    variant = self.get_variant_string(variant_result)
                    simplified_variant_results.append(
                        PhEvalVariantResult(
                            chromosome=self.obtain_chromosome(variant),
                            start=self.obtain_start(variant),
                            end=self.obtain_end(variant),
                            ref=self.obtain_ref(variant),
                            alt=self.obtain_alt(variant),
                            score=self.obtain_score(result),
                        )
                    )
        return simplified_variant_results


class PhEvalDiseaseResultFromLirical:
    def __init__(
        self,
        lirical_result: pd.DataFrame,
    ):
        self.lirical_result = lirical_result

    @staticmethod
    def obtain_disease_identifier(result: pd.Series) -> str:
        """Obtain the disease curie from LIRICAL result."""
        return result["diseaseCurie"]

    @staticmethod
    def obtain_disease_name(result: pd.Series) -> str:
        """Obtain the disease name from LIRICAL result."""
        return result["diseaseName"]

    @staticmethod
    def obtain_score(result: pd.Series) -> float:
        """Obtain score from result."""
        if result["compositeLR"] == "-∞":
            return float("-inf")
        return float(result["compositeLR"])

    def extract_pheval_requirements(self) -> [PhEvalDiseaseResult]:
        """Extract data required to produce PhEval gene output."""
        simplified_disease_results = []
        for _index, result in self.lirical_result.iterrows():
            simplified_disease_results.append(
                PhEvalDiseaseResult(
                    disease_name=self.obtain_disease_name(result),
                    disease_identifier=self.obtain_disease_identifier(result),
                    score=self.obtain_score(result),
                )
            )
        return simplified_disease_results


def create_standardised_results(
    raw_results_dir: Path,
    output_dir: Path,
    sort_order: str,
    disease_analysis: bool,
    gene_analysis: bool,
    variant_analysis: bool,
) -> None:
    """Write standardised gene and variant results from LIRICAL tsv output."""
    identifier_map = create_gene_identifier_map()
    hgnc_data = create_hgnc_dict()
    gene_identifier_updator = GeneIdentifierUpdater(
        gene_identifier="ensembl_id", identifier_map=identifier_map, hgnc_data=hgnc_data
    )
    for result in files_with_suffix(raw_results_dir, ".tsv"):
        lirical_result = read_lirical_result(result)
        if gene_analysis:
            pheval_gene_result = PhEvalGeneResultFromLirical(
                lirical_result, gene_identifier_updator
            ).extract_pheval_requirements()
            generate_pheval_result(pheval_gene_result, sort_order, output_dir, result)
        if variant_analysis:
            pheval_variant_result = PhEvalVariantResultFromLirical(
                lirical_result
            ).extract_pheval_requirements()
            generate_pheval_result(pheval_variant_result, sort_order, output_dir, result)
        if disease_analysis:
            pheval_disease_result = PhEvalDiseaseResultFromLirical(
                lirical_result
            ).extract_pheval_requirements()
            generate_pheval_result(pheval_disease_result, sort_order, output_dir, result)


@click.command("post-process")
@click.option(
    "--results-dir", "-r", required=True, help="Path to LIRICAL results directory.", type=Path
)
@click.option("--output-dir", "-o", required=True, help="Path to output directory.", type=Path)
@click.option(
    "--sort-order",
    "-s",
    required=True,
    help="sort order.",
    type=click.Choice(["ascending", "descending"]),
    default="descending",
    show_default=True,
)
@click.option(
    "--disease-analysis/--no-disease-analysis",
    default=False,
    required=False,
    type=bool,
    show_default=True,
    help="Specify extraction for disease prioritisation results.",
)
@click.option(
    "--gene-analysis/--no-gene-analysis",
    default=False,
    required=False,
    type=bool,
    show_default=True,
    help="Specify analysis for gene prioritisation",
)
@click.option(
    "--variant-analysis/--no-variant-analysis",
    default=False,
    required=False,
    type=bool,
    show_default=True,
    help="Specify analysis for variant prioritisation",
)
def post_process(
    results_dir: Path,
    output_dir: Path,
    sort_order: str,
    disease_analysis: bool,
    gene_analysis: bool,
    variant_analysis: bool,
):
    """Post-process LIRICAL .tsv results to PhEval gene and variant result format."""
    output_dir.joinpath("pheval_gene_results/").mkdir(exist_ok=True, parents=True)
    output_dir.joinpath("pheval_variant_results/").mkdir(exist_ok=True, parents=True)
    create_standardised_results(
        results_dir, output_dir, sort_order, disease_analysis, gene_analysis, variant_analysis
    )



from pathlib import Path
from typing import Optional

from pydantic import BaseModel, Field


class PostProcessing(BaseModel):
    sort_order: str = Field(...)


class ExomiserDB(BaseModel):
    exomiser_database: Optional[Path] = Field(None)
    exomiser_hg19_database: Optional[Path] = Field(None)
    exomiser_hg38_database: Optional[Path] = Field(None)


class LIRICALToolSpecificConfigurations(BaseModel):
    mode: str = Field(...)
    lirical_jar_executable: Path = Field(...)
    exomiser_db_configurations: ExomiserDB = Field(...)
    post_process: PostProcessing = Field(...)



from dataclasses import dataclass
from pathlib import Path


@dataclass
class LiricalPhenopacketCommandLineArguments:
    """Minimal arguments required to run LIRICAL in phenopacket mode on the command line."""

    lirical_jar_file: Path
    phenopacket_path: Path
    vcf_file_path: Path
    assembly: str
    lirical_data: Path
    exomiser_data: Path
    output_dir: Path
    output_prefix: str
    exomiser_hg19_data_path: Path = None
    exomiser_hg38_data_path: Path = None



from pathlib import Path

import click
from packaging import version
from phenopackets import Phenopacket, PhenotypicFeature
from pheval.utils.file_utils import files_with_suffix
from pheval.utils.phenopacket_utils import PhenopacketUtil, phenopacket_reader

from pheval_lirical.prepare.prepare_manual_commands import LiricalManualCommandLineArguments
from pheval_lirical.prepare.prepare_phenopacket_commands import (
    LiricalPhenopacketCommandLineArguments,
)


class CommandCreator:
    def __init__(
        self,
        phenopacket_path: Path,
        phenopacket: Phenopacket,
        lirical_jar: Path,
        input_dir: Path,
        exomiser_data_dir: Path,
        vcf_dir: Path,
        results_dir: Path,
        mode: str,
        exomiser_hg19_data_path: Path,
        exomiser_hg38_data_path: Path,
    ):
        self.phenopacket_path = phenopacket_path
        self.lirical_jar = lirical_jar
        self.input_dir = input_dir
        self.exomiser_data_dir = exomiser_data_dir
        self.vcf_dir = vcf_dir
        self.results_dir = results_dir
        self.mode = mode
        self.exomiser_hg19_data_path = exomiser_hg19_data_path
        self.exomiser_hg38_data_path = exomiser_hg38_data_path
        self.phenopacket_util = PhenopacketUtil(phenopacket)

    def get_list_negated_phenotypic_features(self):
        """Return list of negated HPO ids if there are any present, otherwise return None."""
        return (
            None
            if self.phenopacket_util.negated_phenotypic_features() == []
            else [hpo.type.id for hpo in self.phenopacket_util.negated_phenotypic_features()]
        )

    def get_list_observed_phenotypic_features(self) -> [PhenotypicFeature]:
        """Return list of observed HPO ids."""
        return [hpo.type.id for hpo in self.phenopacket_util.observed_phenotypic_features()]

    def get_vcf_path(self) -> Path:
        """Return the vcf file path."""
        return self.phenopacket_util.vcf_file_data(
            phenopacket_path=self.phenopacket_path, vcf_dir=self.vcf_dir
        ).uri

    def get_vcf_assembly(self) -> str:
        """Return the vcf assembly."""
        return self.phenopacket_util.vcf_file_data(
            phenopacket_path=self.phenopacket_path, vcf_dir=self.vcf_dir
        ).file_attributes["genomeAssembly"]

    def add_manual_cli_arguments(
        self, gene_analysis: bool, variant_analysis: bool
    ) -> LiricalManualCommandLineArguments:
        """Return all CLI arguments to run LIRICAL in manual mode."""
        return LiricalManualCommandLineArguments(
            lirical_jar_file=self.lirical_jar,
            observed_phenotypes=self.get_list_observed_phenotypic_features(),
            negated_phenotypes=self.get_list_negated_phenotypic_features(),
            assembly=self.get_vcf_assembly() if gene_analysis or variant_analysis else None,
            vcf_file_path=self.get_vcf_path() if gene_analysis or variant_analysis else None,
            lirical_data=self.input_dir,
            exomiser_data=self.exomiser_data_dir,
            sample_id=self.phenopacket_util.sample_id(),
            output_dir=self.results_dir,
            output_prefix=self.phenopacket_path.stem,
            exomiser_hg19_data_path=(
                self.exomiser_hg19_data_path if gene_analysis or variant_analysis else None
            ),
            exomiser_hg38_data_path=(
                self.exomiser_hg38_data_path if gene_analysis or variant_analysis else None
            ),
        )

    def add_phenopacket_cli_arguments(
        self, gene_analysis: bool, variant_analysis: bool
    ) -> LiricalPhenopacketCommandLineArguments:
        """Return all CLI arguments to run LIRICAL in phenopacket mode."""
        return LiricalPhenopacketCommandLineArguments(
            lirical_jar_file=self.lirical_jar,
            phenopacket_path=self.phenopacket_path,
            vcf_file_path=self.get_vcf_path() if gene_analysis or variant_analysis else None,
            assembly=self.get_vcf_assembly() if gene_analysis or variant_analysis else None,
            lirical_data=self.input_dir,
            exomiser_data=self.exomiser_data_dir,
            output_dir=self.results_dir,
            output_prefix=self.phenopacket_path.stem,
            exomiser_hg19_data_path=(
                self.exomiser_hg19_data_path if gene_analysis or variant_analysis else None
            ),
            exomiser_hg38_data_path=(
                self.exomiser_hg38_data_path if gene_analysis or variant_analysis else None
            ),
        )

    def add_cli_arguments(
        self, gene_analysis: bool, variant_analysis: bool
    ) -> LiricalManualCommandLineArguments or LiricalManualCommandLineArguments:
        """Return all CLI arguments."""
        if self.mode.lower() == "phenopacket":
            return self.add_phenopacket_cli_arguments(gene_analysis, variant_analysis)
        elif self.mode.lower() == "manual":
            return self.add_manual_cli_arguments(gene_analysis, variant_analysis)


def create_command_arguments(
    phenopacket_dir: Path,
    lirical_jar: Path,
    input_dir: Path,
    exomiser_data_dir: Path,
    vcf_dir: Path,
    output_dir: Path,
    mode: str,
    exomiser_hg19_data: Path,
    exomiser_hg38_data: Path,
    gene_analysis: bool,
    variant_analysis: bool,
) -> list[LiricalManualCommandLineArguments] or list[LiricalPhenopacketCommandLineArguments]:
    """Return a list of LIRICAL command line arguments for a directory of phenopackets."""
    phenopacket_paths = files_with_suffix(phenopacket_dir, ".json")
    commands = []
    for phenopacket_path in phenopacket_paths:
        phenopacket = phenopacket_reader(phenopacket_path)
        commands.append(
            CommandCreator(
                phenopacket_path=phenopacket_path,
                phenopacket=phenopacket,
                lirical_jar=lirical_jar,
                input_dir=input_dir,
                exomiser_data_dir=exomiser_data_dir,
                vcf_dir=vcf_dir,
                results_dir=output_dir,
                mode=mode,
                exomiser_hg19_data_path=exomiser_hg19_data,
                exomiser_hg38_data_path=exomiser_hg38_data,
            ).add_cli_arguments(gene_analysis, variant_analysis)
        )
    return commands


class CommandWriter:
    def __init__(self, mode: str, lirical_version: str, output_file: Path):
        self.mode = mode
        self.version = lirical_version
        self.file = open(output_file, "w")

    def write_java_command(
        self,
        command_arguments: LiricalManualCommandLineArguments
        or LiricalPhenopacketCommandLineArguments,
    ) -> None:
        """Write the basic command do run LIRICAL jar file."""
        self.file.write("java" + " -jar " + str(command_arguments.lirical_jar_file))

    def write_mode(self) -> None:
        """Write mode to run LIRICAL"""
        if self.mode.lower() == "phenopacket":
            self.file.write(" P")
        elif self.mode.lower() == "manual":
            self.file.write(" R")

    def write_phenopacket_path(
        self, command_arguments: LiricalPhenopacketCommandLineArguments
    ) -> None:
        """Write the phenopacket path."""
        self.file.write(" --phenopacket " + str(command_arguments.phenopacket_path))

    def write_observed_phenotypic_features(
        self, command_arguments: LiricalManualCommandLineArguments
    ) -> None:
        """Write observed HPO ids to command."""
        self.file.write(" --observed-phenotypes " + ",".join(command_arguments.observed_phenotypes))

    def write_negated_phenotypic_features(
        self, command_arguments: LiricalManualCommandLineArguments
    ) -> None:
        """Write negated HPO ids to command."""
        if command_arguments.negated_phenotypes is not None:
            self.file.write(
                " --negated-phenotypes " + ",".join(command_arguments.negated_phenotypes)
            )

    def write_vcf_file_properties(
        self,
        command_arguments: LiricalManualCommandLineArguments
        or LiricalPhenopacketCommandLineArguments,
    ) -> None:
        """Write related VCF arguments to command."""
        if command_arguments.vcf_file_path is not None:
            self.file.write(
                " --vcf "
                + str(command_arguments.vcf_file_path)
                + " --assembly "
                + command_arguments.assembly
            )

    def write_sample_id(self, command_arguments: LiricalManualCommandLineArguments) -> None:
        """Write the sample id."""
        self.file.write(" --sample-id " + '"' + command_arguments.sample_id + '"')

    def write_lirical_data_dir(
        self,
        command_arguments: LiricalManualCommandLineArguments
        or LiricalPhenopacketCommandLineArguments,
    ) -> None:
        """Write LIRICAL data directory location."""
        self.file.write(" --data " + str(command_arguments.lirical_data))

    def write_exomiser_data_dir(
        self,
        command_arguments: LiricalManualCommandLineArguments
        or LiricalPhenopacketCommandLineArguments,
    ) -> None:
        """Write Exomiser data location, dealing with deprecated parameters."""
        if version.parse(self.version) > version.parse("2.0.0-RC1"):
            if command_arguments.exomiser_hg19_data_path is not None:
                self.file.write(" -e19 " + str(command_arguments.exomiser_hg19_data_path))
            if command_arguments.exomiser_hg38_data_path is not None:
                self.file.write(" -e38 " + str(command_arguments.exomiser_hg38_data_path))
        if version.parse(self.version) < version.parse("2.0.0-RC2"):
            self.file.write(" --exomiser " + str(command_arguments.exomiser_data))

    def write_output_parameters(self, command_arguments: LiricalManualCommandLineArguments) -> None:
        """Write related output parameter arguments to command."""
        self.file.write(
            " --prefix "
            + command_arguments.output_prefix
            + " --output-directory "
            + str(command_arguments.output_dir)
            + " --output-format "
            + "tsv"
        )

    def write_common_arguments(
        self,
        command_arguments: LiricalManualCommandLineArguments
        or LiricalPhenopacketCommandLineArguments,
    ) -> None:
        """Write common CLI parameters."""
        self.write_java_command(command_arguments)
        self.write_mode()
        self.write_vcf_file_properties(command_arguments)
        self.write_lirical_data_dir(command_arguments)
        self.write_exomiser_data_dir(command_arguments)
        self.write_output_parameters(command_arguments)

    def write_manual_command(self, command_arguments: LiricalManualCommandLineArguments) -> None:
        """Write LIRICAL command to file to run in manual mode."""
        self.write_common_arguments(command_arguments)
        self.write_observed_phenotypic_features(command_arguments)
        self.write_negated_phenotypic_features(command_arguments)
        self.write_sample_id(command_arguments)
        self.file.write("\n")

    def write_phenopacket_command(
        self, command_arguments: LiricalPhenopacketCommandLineArguments
    ) -> None:
        """Write LIRICAL command to file to run in phenopacket mode."""
        self.write_common_arguments(command_arguments)
        self.write_phenopacket_path(command_arguments)
        self.file.write("\n")

    def write_command(
        self,
        command_arguments: LiricalManualCommandLineArguments
        or LiricalPhenopacketCommandLineArguments,
    ) -> None:
        """Write LIRICAL command."""
        try:
            (
                self.write_phenopacket_command(command_arguments)
                if self.mode.lower() == "phenopacket"
                else self.write_manual_command(command_arguments)
            )
        except IOError:
            print("Error writing ", self.file)

    def close(self) -> None:
        """Close file."""
        try:
            self.file.close()
        except IOError:
            print("Error closing ", self.file)


def write_all_commands(
    command_arguments: [LiricalManualCommandLineArguments]
    or [LiricalPhenopacketCommandLineArguments],
    tool_input_commands_dir: Path,
    file_prefix: Path,
    mode: str,
    lirical_version: str,
) -> None:
    """Write all commands to file for running LIRICAL."""
    command_writer = CommandWriter(
        mode=mode,
        lirical_version=lirical_version,
        output_file=tool_input_commands_dir.joinpath(f"{file_prefix}-lirical-commands.txt"),
    )
    for command_argument in command_arguments:
        command_writer.write_command(command_argument)
    command_writer.close()


def prepare_commands(
    lirical_jar: Path,
    input_dir: Path,
    exomiser_data_dir: Path,
    phenopacket_dir: Path,
    vcf_dir: Path,
    file_prefix: str,
    tool_input_commands_dir: Path,
    raw_results_dir: Path,
    mode: str,
    lirical_version: str,
    exomiser_hg19_data: Path,
    exomiser_hg38_data: Path,
    gene_analysis: bool,
    variant_analysis: bool,
) -> None:
    """Prepare command batch files to run LIRICAL."""
    command_arguments = create_command_arguments(
        phenopacket_dir,
        lirical_jar,
        input_dir,
        exomiser_data_dir,
        vcf_dir,
        raw_results_dir,
        mode,
        exomiser_hg19_data,
        exomiser_hg38_data,
        gene_analysis,
        variant_analysis,
    )
    write_all_commands(
        command_arguments, tool_input_commands_dir, file_prefix, mode, lirical_version
    )


@click.command("prepare-commands")
@click.option("--lirical-jar", "-l", required=True, help="Path to Lirical jar file.", type=Path)
@click.option("--input-dir", "-i", required=True, help="Path to Lirical data directory.", type=Path)
@click.option(
    "--exomiser-data-dir",
    "-exomiser",
    required=False,
    help="Path to exomiser data directory.",
    type=Path,
)
@click.option("--phenopacket-dir", "-p", required=True, help="Path to phenopacket.", type=Path)
@click.option("--vcf-dir", "-v", required=True, help="Path to vcf directory.", type=Path)
@click.option("--file-prefix", "-f", required=True, help="File prefix", type=Path)
@click.option("--output-dir", "-o", required=True, help="Path to output of batch files.", type=Path)
@click.option(
    "--results-dir", "-r", required=True, help="Path to output LIRICAL results.", type=Path
)
@click.option(
    "--mode",
    "-m",
    required=False,
    default="manual",
    help="Mode to run LIRICAL.",
    type=click.Choice(["phenopacket", "manual"]),
)
@click.option("--lirical_version", "-v", required=True, help="Lirical version.", type=str)
@click.option(
    "--exomiser-hg19", "-e19", required=False, help="Exomiser hg19 variant database.", type=Path
)
@click.option(
    "--exomiser-hg38", "-e38", required=False, help="Exomiser hg38 variant database.", type=Path
)
@click.option(
    "--gene-analysis/--no-gene-analysis",
    default=False,
    required=False,
    type=bool,
    show_default=True,
    help="Specify analysis for gene prioritisation",
)
@click.option(
    "--variant-analysis/--no-variant-analysis",
    default=False,
    required=False,
    type=bool,
    show_default=True,
    help="Specify analysis for variant prioritisation",
)
def prepare_commands_command(
    lirical_jar: Path,
    input_dir: Path,
    exomiser_data_dir: Path,
    phenopacket_dir: Path,
    vcf_dir: Path,
    file_prefix: str,
    output_dir: Path,
    results_dir: Path,
    mode: str,
    lirical_version: str,
    exomiser_hg19: Path,
    exomiser_hg38: Path,
    gene_analysis: bool,
    variant_analysis: bool,
):
    """Prepare command batch files to run LIRICAL."""
    output_dir.joinpath("tool_input_commands").mkdir(parents=True, exist_ok=True)
    prepare_commands(
        lirical_jar,
        input_dir,
        exomiser_data_dir,
        phenopacket_dir,
        vcf_dir,
        file_prefix,
        output_dir.joinpath("tool_input_commands"),
        results_dir,
        mode,
        lirical_version,
        exomiser_hg19,
        exomiser_hg38,
        gene_analysis,
        variant_analysis,
    )



import subprocess
from pathlib import Path

from pheval.utils.file_utils import all_files

from pheval_lirical.prepare.prepare_commands import prepare_commands
from pheval_lirical.tool_specific_configuration_parser import LIRICALToolSpecificConfigurations


def prepare_lirical_commands(
    input_dir: Path,
    tool_input_commands_dir: Path,
    raw_results_dir: Path,
    testdata_dir: Path,
    lirical_version: str,
    tool_specific_configurations: LIRICALToolSpecificConfigurations,
    gene_analysis: bool,
    variant_analysis: bool,
):
    """Write commands to run LIRICAL."""
    phenopacket_dir = Path(testdata_dir).joinpath("phenopackets")
    vcf_dir = Path(testdata_dir).joinpath("vcf") if gene_analysis or variant_analysis else None
    prepare_commands(
        lirical_jar=input_dir.joinpath(tool_specific_configurations.lirical_jar_executable),
        input_dir=input_dir.joinpath("data"),
        exomiser_data_dir=(
            input_dir.joinpath(
                tool_specific_configurations.exomiser_db_configurations.exomiser_database
            )
            if tool_specific_configurations.exomiser_db_configurations.exomiser_database is not None
            else None
        ),
        phenopacket_dir=phenopacket_dir,
        vcf_dir=vcf_dir,
        file_prefix=Path(testdata_dir).name,
        tool_input_commands_dir=tool_input_commands_dir,
        raw_results_dir=raw_results_dir,
        mode=tool_specific_configurations.mode,
        lirical_version=lirical_version,
        exomiser_hg19_data=(
            input_dir.joinpath(
                tool_specific_configurations.exomiser_db_configurations.exomiser_hg19_database
            )
            if tool_specific_configurations.exomiser_db_configurations.exomiser_hg19_database
            else None
        ),
        exomiser_hg38_data=(
            input_dir.joinpath(
                tool_specific_configurations.exomiser_db_configurations.exomiser_hg38_database
            )
            if tool_specific_configurations.exomiser_db_configurations.exomiser_hg38_database
            is not None
            else None
        ),
        gene_analysis=gene_analysis,
        variant_analysis=variant_analysis,
    ),


def run_lirical_local(tool_input_commands_dir: Path, testdata_dir: Path):
    """Run LIRICAL locally."""
    batch_file = [
        file
        for file in all_files(Path(tool_input_commands_dir))
        if file.name.startswith(Path(testdata_dir).name)
    ][0]
    print("running LIRICAL")
    subprocess.run(
        ["bash", str(batch_file)],
        shell=False,
    )



from pathlib import Path

from pheval_lirical.post_process.post_process_results_format import create_standardised_results
from pheval_lirical.tool_specific_configuration_parser import LIRICALToolSpecificConfigurations


def post_process_results_format(
    raw_results_dir: Path,
    output_dir: Path,
    config: LIRICALToolSpecificConfigurations,
    disease_analysis: bool,
    gene_analysis: bool,
    variant_analysis: bool,
):
    """Create pheval gene and variant result from LIRICAL tsv output."""
    print("...creating pheval results format...")
    create_standardised_results(
        raw_results_dir=raw_results_dir,
        output_dir=output_dir,
        sort_order=config.post_process.sort_order,
        disease_analysis=disease_analysis,
        gene_analysis=gene_analysis,
        variant_analysis=variant_analysis,
    )
    print("done")



from dataclasses import dataclass
from pathlib import Path

from phenopackets import PhenotypicFeature


@dataclass
class LiricalManualCommandLineArguments:
    """Minimal arguments required to run LIRICAL manually on the command line."""

    lirical_jar_file: Path
    observed_phenotypes: [PhenotypicFeature]
    negated_phenotypes: [PhenotypicFeature] or None
    assembly: str
    vcf_file_path: Path
    sample_id: str
    lirical_data: Path
    exomiser_data: Path
    output_dir: Path
    output_prefix: str
    exomiser_hg19_data_path: Path = None
    exomiser_hg38_data_path: Path = None
